use std::future::Future;
use std::io;

use slog::Logger;

use smol::net::TcpStream;

use capnp_rpc::twoparty;
use capnp_rpc::RpcSystem;
use capnp_rpc::rpc_twoparty_capnp::Side;

mod auth_capnp {
    include!(concat!(env!("OUT_DIR"), "/schema/auth_capnp.rs"));
}
mod connection_capnp {
    include!(concat!(env!("OUT_DIR"), "/schema/connection_capnp.rs"));
}
mod api_capnp {
    include!(concat!(env!("OUT_DIR"), "/schema/api_capnp.rs"));
}

mod api;
pub use api::API;

mod authentication;
pub use authentication::Authentication;

pub fn bootstrap(log: Logger, stream: TcpStream) -> (RpcSystem<Side>, API) {
    let network = Box::new(twoparty::VatNetwork::new(stream.clone(), stream,
        Side::Client, Default::default()));

    let mut rpc = RpcSystem::new(network, None);
    let client: connection_capnp::bootstrap::Client 
        = rpc.bootstrap(Side::Server);

    return (rpc, API::new(log, client));
}

async fn handshake(log: &Logger, mut stream: &mut TcpStream) -> Result<(), io::Error> {
    let host = "localhost";
    let program = format!("{}-{}", env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION"));
    let version = (0u32,1u32);

    debug!(log, "Sending handshake as {}@{} speaking API v{}.{}", 
        program, host, version.0, version.1);

    let mut message = capnp::message::Builder::new_default();
    let mut builder = message.init_root::<connection_capnp::greeting::Builder>();

    builder.set_host(host);
    builder.set_major(version.0);
    builder.set_minor(version.1);
    builder.set_program(&program);

    capnp_futures::serialize::write_message(&mut stream, message).await.unwrap();
    if let Some(m) = capnp_futures::serialize::read_message(&mut stream, Default::default()).await.unwrap() {
        let greeting = m.get_root::<connection_capnp::greeting::Reader>().unwrap();
        let peer_host = greeting.get_host().unwrap();
        let peer_program = greeting.get_program().unwrap();
        let major = greeting.get_major();
        let minor = greeting.get_minor();

        info!(log, "Peer {} running {} API {}.{}", peer_host, peer_program, major, minor)
    } else {
        error!(log, "Oh noes");
    }

    Ok(())
}
